/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven;

import java.util.HashMap;
import java.util.Map;

import org.sonar.commons.database.DatabaseSession;
import org.sonar.commons.measures.Measure;
import org.sonar.commons.measures.MeasureParameter;
import org.sonar.plugins.api.maven.ProjectAnalysis;
import org.sonar.plugins.api.maven.Resource;
import org.sonar.plugins.api.maven.model.MavenPom;

import ch.hortis.sonar.model.MavenProject;
import ch.hortis.sonar.model.Metric;
import ch.hortis.sonar.model.ResourcesDao;
import ch.hortis.sonar.model.Rule;
import ch.hortis.sonar.model.RuleFailure;
import ch.hortis.sonar.model.RuleFailureLevel;
import ch.hortis.sonar.model.RuleFailureParam;
import ch.hortis.sonar.model.Snapshot;
import ch.hortis.sonar.model.SnapshotSource;
import ch.hortis.sonar.service.MeasuresDao;

public class ProjectAnalysisImpl implements ProjectAnalysis {
  private DatabaseSession session;

  private Map<Resource, Snapshot> resourceToSnapshot;
  private Map<Resource, MavenProject> resourceToModel;

  private Snapshot projectSnapshot;
  private MavenProject project;

  public ProjectAnalysisImpl(DatabaseSession session) {
    this.resourceToSnapshot = new HashMap<Resource, Snapshot>();
    this.resourceToModel = new HashMap<Resource, MavenProject>();
    this.session = session;
  }

  protected Snapshot init(MavenPom pom) {
    if (projectSnapshot != null) {
      return projectSnapshot;
    }
    project = loadProject(pom);

    projectSnapshot = new Snapshot();
    projectSnapshot.setMavenProject(project);
    projectSnapshot.setVersion(pom.getAnalysisVersion());
    projectSnapshot.setCreatedAt(pom.getAnalysisDate());
    projectSnapshot = (Snapshot) session.save(projectSnapshot);
    return projectSnapshot;
  }

  private MavenProject loadProject(MavenPom pom) {
    ResourcesDao dao = new ResourcesDao(session);
    MavenProject result = dao.getResource(pom.getKey(), MavenProject.SCOPE_PROJECT);
    if (result == null) {
      result = new MavenProject();
      result.setScope(MavenProject.SCOPE_PROJECT);
      result.setKey(pom.getKey());
      result.setQualifier(pom.getLanguage().getProjectQualifier());
    }

    result.setName(pom.getName());
    result.setDescription(pom.getDescription());
    result = (MavenProject) session.save(result);
    return result;
  }

  public void addMeasure(Metric metric, Double value, MeasureParameter... params) {
    addMeasure(projectSnapshot, metric, value, null, params);
  }

  public void addMeasure(Metric metric, Double value, String textValue, MeasureParameter... params) {
    addMeasure(projectSnapshot, metric, value, textValue, params);
  }

  public void addMeasure(Resource resource, Metric metric, Double value, MeasureParameter... params) {
    addMeasure(resource, metric, value, null, params);
  }

  public void addMeasure(Resource resource, Metric metric, Double value, String textValue, MeasureParameter... params) {
    Snapshot snapshot = saveSnapshot(resource);
    addMeasure(snapshot, metric, value, textValue, params);
  }

  public void addViolation(Resource resource, Rule rule, String message, RuleFailureLevel level, RuleFailureParam... params) {
    RuleFailure failure = new RuleFailure(rule, level);
    failure.setMessage(message);
    failure.setSnapshot(reattach(saveSnapshot(resource)));
    failure = (RuleFailure) session.save(failure);

    for (RuleFailureParam param : params) {
      if (param != null) {
        param.setRuleFailure(failure);
        param.setSnapshot(reattach(saveSnapshot(resource)));
        failure.getParameters().add(param);
        session.save(param);
      }
    }
  }

  public void addSource(Resource resource, String source) {
    Snapshot snapshot = reattach(saveSnapshot(resource));
    session.save(new SnapshotSource(snapshot, source));
  }

  private Measure addMeasure(Snapshot snapshot, Metric metric, Double value, String textValue, MeasureParameter... params) {
    MeasuresDao dao = new MeasuresDao(session);
    Metric dbMetric = dao.getMetric(metric);
    Measure measure = (Measure) session.save(new Measure(reattach(snapshot), dbMetric, value).setTextValue(textValue));
    if (params != null) {
      for (MeasureParameter parameter : params) {
        parameter.setMeasure(measure);
        parameter.setSnapshot(snapshot);
        measure.getParams().add(parameter);
        session.save(parameter);
      }
    }
    return measure;
  }

  private Snapshot saveSnapshot(Resource resource) {
    if (resource == null) {
      return null;
    }
    Snapshot parent = saveSnapshot(resource.getParent());
    Snapshot snapshot = resourceToSnapshot.get(resource);
    if (snapshot == null) {
      snapshot = new Snapshot();
      snapshot.setMavenProject(reattach(saveResource(resource)));
      snapshot.setCreatedAt(projectSnapshot.getCreatedAt());
      Snapshot root = reattach(projectSnapshot);
      snapshot.setRoot(root);
      if (parent == null) {
        snapshot.setParent(root);
      } else {
        snapshot.setParent(reattach(parent));
      }
      snapshot = (Snapshot) session.save(snapshot);
      resourceToSnapshot.put(resource, snapshot);
    }
    return snapshot;
  }

  private MavenProject saveResource(Resource resource) {
    MavenProject row = resourceToModel.get(resource);
    if (row == null) {
      String fullkey = getFullKey(resource);
      row = session.getSingleResult(MavenProject.class,
          "scope", resource.getDbScope(), "key", fullkey, "rootId", project.getId(), "enabled", true);
      if (row == null) {
        row = new MavenProject(resource.getDbScope(), fullkey, resource.getDbQualifier(), project.getId(), resource.getName());
        row = (MavenProject) session.save(row);
      }
      resourceToModel.put(resource, row);
    }
    return row;
  }

  private String getFullKey(Resource resource) {
    return new StringBuffer().append(project.getKey()).append(':').append(resource.getKey()).toString();
  }

  private MavenProject reattach(MavenProject row) {
    return session.reattach(MavenProject.class, row.getId());
  }

  private Snapshot reattach(Snapshot row) {
    return session.reattach(Snapshot.class, row.getId());
  }
}